@@ -15,9 +15,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "gtksourcecompletionwordslibrary.h"
@@ -28,7 +27,7 @@ enum
 {
 	LOCK,
 	UNLOCK,
-	NUM_SIGNALS
+	N_SIGNALS
 };
 
 struct _GtkSourceCompletionWordsLibraryPrivate
@@ -37,7 +36,7 @@ struct _GtkSourceCompletionWordsLibraryPrivate
 	gboolean locked;
 };
 
-static guint signals[NUM_SIGNALS] = {0,};
+static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceCompletionWordsLibrary, gtk_source_completion_words_library, G_TYPE_OBJECT)
 
