@@ -15,13 +15,12 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_SOURCE_COMPLETION_WORDS_UTILS_H__
-#define __GTK_SOURCE_COMPLETION_WORDS_UTILS_H__
+#ifndef GTK_SOURCE_COMPLETION_WORDS_UTILS_H
+#define GTK_SOURCE_COMPLETION_WORDS_UTILS_H
 
 #include <gtk/gtk.h>
 
@@ -38,6 +37,10 @@ G_GNUC_INTERNAL
 void		 _gtk_source_completion_words_utils_adjust_region	(GtkTextIter *start,
 									 GtkTextIter *end);
 
+G_GNUC_INTERNAL
+void		 _gtk_source_completion_words_utils_check_scan_region	(const GtkTextIter *start,
+									 const GtkTextIter *end);
+
 G_END_DECLS
 
-#endif /* __GTK_SOURCE_COMPLETION_WORDS_UTILS_H__ */
+#endif /* GTK_SOURCE_COMPLETION_WORDS_UTILS_H */
