@@ -18,9 +18,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -2053,12 +2052,6 @@ gtk_source_buffer_create_source_mark (GtkSourceBuffer   *buffer,
 				  GTK_TEXT_MARK (mark),
 				  where);
 
-	/* We want to return a borrowed reference and the mark is already
-	 * owned by @buffer due to gtk_text_buffer_add_mark(). Therefore
-	 * it is safe to unref immediately.
-	 */
-	g_object_unref (mark);
-
 	return mark;
 }
 
