@@ -14,9 +14,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -889,7 +888,7 @@ begin_draw (GtkSourceGutter *gutter,
 	    LinesInfo       *info,
 	    cairo_t         *cr)
 {
-	GdkRectangle background_area;
+	GdkRectangle background_area = { 0 };
 	GdkRectangle cell_area;
 	GList *l;
 	gint renderer_num;
