@@ -14,17 +14,17 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_SOURCE_GUTTER_RENDERER_LINES_H__
-#define __GTK_SOURCE_GUTTER_RENDERER_LINES_H__
+#ifndef GTK_SOURCE_GUTTER_RENDERER_LINES_H
+#define GTK_SOURCE_GUTTER_RENDERER_LINES_H
 
-#include <gtksourceview/gtksourcetypes.h>
-#include <gtksourceview/gtksourcegutterrenderertext.h>
 #include <gtk/gtk.h>
+#include "gtksourcetypes.h"
+#include "gtksourcetypes-private.h"
+#include "gtksourcegutterrenderertext.h"
 
 G_BEGIN_DECLS
 
@@ -57,10 +57,12 @@ struct _GtkSourceGutterRendererLinesClass
 	/*< public >*/
 };
 
+G_GNUC_INTERNAL
 GType gtk_source_gutter_renderer_lines_get_type (void) G_GNUC_CONST;
 
+G_GNUC_INTERNAL
 GtkSourceGutterRenderer *gtk_source_gutter_renderer_lines_new (void);
 
 G_END_DECLS
 
-#endif /* __GTK_SOURCE_GUTTER_RENDERER_LINES_H__ */
+#endif /* GTK_SOURCE_GUTTER_RENDERER_LINES_H */
