@@ -14,9 +14,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -200,8 +199,6 @@ gutter_renderer_query_data (GtkSourceGutterRenderer      *renderer,
 	              "xalign", 0.5,
 	              "alignment-mode", GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST,
 	              NULL);
-
-	g_clear_object (&pixbuf);
 }
 
 static gboolean
@@ -337,7 +334,6 @@ gutter_renderer_query_tooltip (GtkSourceGutterRenderer *renderer,
 	GSList *marks;
 	GtkSourceView *view;
 	GtkSourceBuffer *buffer;
-	gboolean ret;
 
 	view = GTK_SOURCE_VIEW (gtk_source_gutter_renderer_get_view (renderer));
 	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
@@ -354,11 +350,7 @@ gutter_renderer_query_tooltip (GtkSourceGutterRenderer *renderer,
 
 		marks = g_slist_reverse (marks);
 
-		ret = set_tooltip_widget_from_marks (view, tooltip, marks);
-
-		g_slist_free (marks);
-
-		return ret;
+		return set_tooltip_widget_from_marks (view, tooltip, marks);
 	}
 
 	return FALSE;
