@@ -16,9 +16,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -55,7 +54,7 @@
 #define PARSER_ERROR (parser_error_quark ())
 #define ATTR_NO_STYLE ""
 
-typedef enum {
+typedef enum _ParserError {
 	PARSER_ERROR_CANNOT_OPEN     = 0,
 	PARSER_ERROR_CANNOT_VALIDATE,
 	PARSER_ERROR_INVALID_DOC,
@@ -947,7 +946,7 @@ replace_by_id (const GMatchInfo *match_info,
 	if (subst == NULL)
 		g_set_error (&tmp_error,
 			     PARSER_ERROR, PARSER_ERROR_WRONG_ID,
-			     _("Unknown id '%s' in regex '%s'"), id,
+			     _("Unknown id “%s” in regex “%s”"), id,
 			     g_match_info_get_string (match_info));
 
 	if (tmp_error == NULL)
@@ -1177,7 +1176,7 @@ expand_regex (ParserState *parser_state,
 		if (g_regex_get_max_backref (compiled) > 0)
 		{
 			g_set_error (error, PARSER_ERROR, PARSER_ERROR_MALFORMED_REGEX,
-				     _("in regex '%s': backreferences are not supported"),
+				     _("in regex “%s”: backreferences are not supported"),
 				     regex);
 			g_regex_unref (compiled);
 			return NULL;
