@@ -18,9 +18,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -91,7 +90,7 @@ static GTimer *pagination_timer = NULL;
 /* Number of pages paginated on each invocation of the paginate() method. */
 #define PAGINATION_CHUNK_SIZE 3
 
-typedef enum
+typedef enum _PaginatorState
 {
 	/* Initial state: properties can be changed only when the paginator
 	   is in the INIT state */
