@@ -16,9 +16,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <string.h>
@@ -979,20 +978,6 @@ test_widget_class_init (TestWidgetClass *klass)
 	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, background_pattern);
 }
 
-static gboolean
-tranform_boolean_to_draw_spaces_flags (GBinding *binding,
-                                       const GValue *from_value,
-                                       GValue *to_value,
-                                       gpointer user_data)
-{
-	gboolean active;
-
-	active = g_value_get_boolean (from_value);
-	g_value_set_flags (to_value, active ? GTK_SOURCE_DRAW_SPACES_ALL : 0);
-
-	return TRUE;
-}
-
 static void
 show_top_border_window_toggled_cb (GtkToggleButton *checkbutton,
 				   TestWidget      *self)
@@ -1009,6 +994,8 @@ show_top_border_window_toggled_cb (GtkToggleButton *checkbutton,
 static void
 test_widget_init (TestWidget *self)
 {
+	GtkSourceSpaceDrawer *space_drawer;
+
 	self->priv = test_widget_get_instance_private (self);
 
 	gtk_widget_init_template (GTK_WIDGET (self));
@@ -1067,16 +1054,6 @@ test_widget_init (TestWidget *self)
 	                        "visible",
 	                        G_BINDING_SYNC_CREATE);
 
-	g_object_bind_property_full (self->priv->draw_spaces_checkbutton,
-	                             "active",
-	                             self->priv->view,
-	                             "draw-spaces",
-	                             G_BINDING_SYNC_CREATE,
-	                             tranform_boolean_to_draw_spaces_flags,
-	                             NULL,
-	                             NULL,
-	                             NULL);
-
 	g_object_bind_property (self->priv->smart_backspace_checkbutton,
 	                        "active",
 	                        self->priv->view,
@@ -1088,6 +1065,11 @@ test_widget_init (TestWidget *self)
 	                  G_CALLBACK (on_background_pattern_changed),
 	                  self);
 
+	space_drawer = gtk_source_view_get_space_drawer (self->priv->view);
+	g_object_bind_property (self->priv->draw_spaces_checkbutton, "active",
+				space_drawer, "enable-matrix",
+				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+
 	open_file (self, TOP_SRCDIR "/gtksourceview/gtksourcebuffer.c");
 }
 
